%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
%
% ControlParameters.m
%
% Project:	"Marine Cybernetics Simulator: MCSim Hybrid Sensor and Model Based"
%
% Abstract:	Initialize simulation and start Simulink
% 
% Inputs:	-
%
% Outputs:	-
%	
%
% Author:       Svenn Are Vrn and Astrid Brodtkorb, Autumn 2015
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 



Control.EnableRefFF = 1;   % 1 on, 0 off
Control.EnablePDControl = 1; 
Control.EnableIaction = 1; 

%%  controller tuning(s)



Control.Vessel.M = [Par.Vessel.M(1,1) Par.Vessel.M(1,2) Par.Vessel.M(1,6);
                         Par.Vessel.M(2,1) Par.Vessel.M(2,2) Par.Vessel.M(2,6);
                         Par.Vessel.M(6,1) Par.Vessel.M(6,2) Par.Vessel.M(6,6)];
                     
Control.Vessel.D     = [Par.Vessel.DL(1,1) Par.Vessel.DL(1,2) Par.Vessel.DL(1,6);
                        Par.Vessel.DL(2,1) Par.Vessel.DL(2,2) Par.Vessel.DL(2,6);
                        Par.Vessel.DL(6,1) Par.Vessel.DL(6,2) Par.Vessel.DL(6,6)];

                    
% if supply.m:
if Sim.vessno == 1                  
    
     
    Control.Tau_max					= 10e9*[1 1 10];	% Max thrust vector

    Control.PID.Ki_sat				= 0.8*Control.Tau_max;	% Saturation of integral term
    
      
    
    %******************************
    % PID DP settings
    %Control.PID.zeta				= [1.2 1.2 1.2];	% Design damping ratio [-] [surge sway yaw]
    Control.PID.zeta				= [2 2 2];	% Design damping ratio [-] [surge sway yaw]
    Control.PID.T_n					= [80 80 100];	% Design time constant [s] [surge sway yaw]
    Control.PID.K_m					= [0 0 0];		% Acceleration feedback constants [surge sway yaw]
    Control.PID.T_i_fac				= [11 7 7];	% Factor by which the integral time constant is slower than the natural frequency
    
    
    Control.PID.M					= diag(Control.Vessel.M)';	% 3DOF mass matrix main diagonal values from vessel model
    Control.PID.D					= diag(Control.Vessel.D)';	% 3DOF linear damping matrix main diagonal values from vessel model
    
    % Controller design
    Control.PID.omega_n				= 2*pi./Control.PID.T_n;
    Control.PID.Kp					= 1*diag((Control.PID.M + Control.PID.K_m).*Control.PID.omega_n.^2);	% 0.5								% Proportional gain vector
    Control.PID.Kd					= 0.5*diag((Control.PID.M + Control.PID.K_m)*2.*Control.PID.zeta.*Control.PID.omega_n - Control.PID.D); % 0.05	% Derivative gain vector
    Control.PID.Ki					= diag(Control.PID.omega_n*Control.PID.Kp/diag(Control.PID.T_i_fac));										% Integral gain vector   % diag([600,600, 1.1315e5]);  %  NOLCOS   
    
    if Sim.Observer == 0
        Control.PID.Kp					= 1*diag((Control.PID.M + Control.PID.K_m).*Control.PID.omega_n.^2);	% 0.5								% Proportional gain vector
        Control.PID.Kd					= 0.7*diag((Control.PID.M + Control.PID.K_m)*2.*Control.PID.zeta.*Control.PID.omega_n - Control.PID.D); % 0.05	% Derivative gain vector
        Control.PID.Ki					= 0.7*diag(Control.PID.omega_n*Control.PID.Kp/diag(Control.PID.T_i_fac));										% Integral gain vector   % diag([600,600, 1.1315e5]);  %  NOLCOS
    end
    
    % decoupled matrices for the feedforward part(only in xy)
    Control.Decoupled.M = diag(Control.PID.M);
    Control.Decoupled.D = diag(Control.PID.D);
    Control.Decoupled.Mxy = Control.Decoupled.M(1:2,1:2);
    Control.Decoupled.Dxy = Control.Decoupled.D(1:2,1:2);
    

    
% if CS.m:
elseif Sim.vessno == 2


    Control.Tau_max					= [10 10 12];	% Max thrust vector
    Control.PID.Ki_sat				= 0.8*Control.Tau_max;	% Saturation of integral term

    
    %******************************
    % PID DP settings
    %Control.PID.zeta				= [1.2 1.2 1.2];	% Design damping ratio [-] [surge sway yaw]
    Control.PID.zeta				= [1.2 1.2 1.2];	% Design damping ratio [-] [surge sway yaw]
    Control.PID.T_n					= [20 20 15];	% Design time constant [s] [surge sway yaw]
    Control.PID.K_m					= [0 0 0];		% Acceleration feedback constants [surge sway yaw]
    Control.PID.T_i_fac				= [11 7 7];	% Factor by which the integral time constant is slower than the natural frequency

    
    Control.PID.M					= diag(Control.Vessel.M)';	% 3DOF mass matrix main diagonal values from vessel model
    Control.PID.D					= diag(Control.Vessel.D)';	% 3DOF linear damping matrix main diagonal values from vessel model
    
    % Controller design
    Control.PID.omega_n				= 2*pi./Control.PID.T_n;
    Control.PID.Kp					= 1*diag((Control.PID.M + Control.PID.K_m).*Control.PID.omega_n.^2);	% 0.5								% Proportional gain vector
    Control.PID.Kd					= 0.3*diag((Control.PID.M + Control.PID.K_m)*2.*Control.PID.zeta.*Control.PID.omega_n - Control.PID.D); % 0.05	% Derivative gain vector
    Control.PID.Ki					= diag(Control.PID.omega_n*Control.PID.Kp/diag(Control.PID.T_i_fac));										% Integral gain vector
    %Control.PID.AW                  = Control.PID.Kp/Control.PID.Ki;                                                                            % antiwindup parameter
    Control.PID.AW                  = diag([1 1 1]);
    
    % decoupled matrices for the feedforward part(only in xy)
    Control.Vessel.M = diag(Control.PID.M);
    Control.Vessel.D = diag(Control.PID.D);
    Control.Vessel.Mxy = Control.Vessel.M(1:2,1:2);
    Control.Vessel.Dxy = Control.Vessel.D(1:2,1:2);
    


%     Control.PID.Kp = 0.5*diag([3.8714    7.5319    1.4955]);
%     Control.PID.Kd = 0*diag([11.9495   28.4064    4.1977]);
%     Control.PID.Ki = 0*diag([0.2068    0.4024    0.0746]);
  
end

  % Quay Control
    Control.Quay.bias   = 0.1*Control.Tau_max; % [N]
    
    
    