%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
% ObserverParameters.m
%
% Project:	"Marine Cybernetics Simulator: MCSim Hybrid Sensor and Model Based"
%
% Abstract:	Observer settings
%
% Inputs:	-
%
% Outputs:	-
%
%
% Author:       Svenn Are Vrn and Astrid Brodtkorb, Autumn 2015
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++








%% NPO settings
%******************************
Control.VessObs.Cw = [0 0 0 1 0 0;
    0 0 0 0 1 0;
    0 0 0 0 0 1];

Obs.NPO.T = [1000 0 0; 0 1000 0; 0 0 1000];
Obs.NPO.T_inv = inv(Obs.NPO.T);


% Inertia and damp matrix for 3DOF (Surge, sway and yaw)

Obs.NPO.M_inv = [Par.Vessel.M_inv(1,1) Par.Vessel.M_inv(1,2) Par.Vessel.M_inv(1,6);
    Par.Vessel.M_inv(2,1) Par.Vessel.M_inv(2,2) Par.Vessel.M_inv(2,6);
    Par.Vessel.M_inv(6,1) Par.Vessel.M_inv(6,2) Par.Vessel.M_inv(6,6)];

Obs.NPO.M  = [Par.Vessel.M(1,1) 0 0 ;...
    0 Par.Vessel.M(2,2) 0;
    0 0 Par.Vessel.M(6,6)];

Obs.NPO.D     = [Par.Vessel.DL(1,1) Par.Vessel.DL(1,2) Par.Vessel.DL(1,6);
    Par.Vessel.DL(2,1) Par.Vessel.DL(2,2) Par.Vessel.DL(2,6);
    Par.Vessel.DL(6,1) Par.Vessel.DL(6,2) Par.Vessel.DL(6,6)];

%******************************
% Inertia and damping matrix for 3DOF (Surge, sway and yaw)

if Sim.vessno == 1
    
    
    Obs.NPO.lmb   = 0.1;
    Obs.NPO.omg_0 = Par.Wave.omega_peak(1);
    Obs.NPO.damp = 1;
    Obs.NPO.k     = 1.22;
    M_obs = inv(Obs.NPO.M_inv);
    Obs.NPO.omg_c = Obs.NPO.k*Obs.NPO.omg_0;  % cut-off frequency
    
    M_obs = diag(diag(M_obs));
    
    % for extreme seas
    %Obs.NPO.K4 = 25*M_obs*[1 0 0; 0 1 0; 0 0 3]*1;
    %Obs.NPO.K3 = 0.001*[Obs.NPO.K4(1,1) 0 0; 0 Obs.NPO.K4(2,2) 0; 0 0 Obs.NPO.K4(3,3)];
    
    
    % for normal seas:
    
    Obs.NPO.K4 = 0.03*M_obs*[1 0 0; 0 1 0; 0 0 3]*1;
    Obs.NPO.K3 = 0.02*[Obs.NPO.K4(1,1) 0 0; 0 Obs.NPO.K4(2,2) 0; 0 0 Obs.NPO.K4(3,3)];
    NPO.mult_low_waves = 3.5; %multiplication of the K2-gain (position) when there are long waves
    %set to 1 if the normal wave tuning is wanted
    NPO.low_waves_threshold = 0.4; %if w_p is above this, then the K2 gain depending on w_c is increased 
    % \by the factor NPO.mult_low_waves
    NPO.no_waves_threshold = 0.3; %if w_p is above this, then the K2 gain depending on w_c is used
    % and if below, then a constant K2 is used
    K2_no_waves = 1*eye(3);
    
    L_bias_eps_NPO = .1*[1.1;1.1; 0.005];
    L_ScaleGain = [20e3,20e3,10000e3]; %gain to scale the feedback in the dynamics for L_bias_gain
    L_biasGain_min_NPO = (Obs.NPO.K3);
    L_biasGain_max_NPO = L_biasGain_min_NPO*1.01;
    
    
    
    
% %% Lower NPO tuning to get better steady state behavior
% Obs.NPO.K4 = 0.02*M_obs*[1 0 0; 0 1 0; 0 0 3]*1;  % on the velocity dynamics
% Obs.NPO.K3 = 0.01*[Obs.NPO.K4(1,1) 0 0; 0 Obs.NPO.K4(2,2) 0; 0 0 Obs.NPO.K4(3,3)];  % on the bias dynamics
% L_bias_eps_NPO = .1*[1.1;1.1; 0.005];
% L_ScaleGain = [1 1 1];  %[20e3,20e3,10000e3]; %gain to scale the feedback in the dynamics for L_bias_gain
% L_biasGain_min_NPO = (Obs.NPO.K3);
% L_biasGain_max_NPO = L_biasGain_min_NPO*1.01;
  %%  
    
    % NPO with velocity estimate from Grip
    Obs.NPO.K41 = 0.02*M_obs*[1 0 0; 0 1 0; 0 0 3]*1;   % 0.02   % velocity
    Obs.NPO.K31 = 0.01*[Obs.NPO.K41(1,1) 0 0; 0 Obs.NPO.K41(2,2) 0; 0 0 Obs.NPO.K41(3,3)]; % 0.2   bias
    
    Obs.NPO.K22= Obs.NPO.omg_c*diag([1 1 1]);  % position
    Obs.NPO.K32 = 2*Obs.NPO.K31*diag([1 1 1]);  % bias
    Obs.NPO.K42 = 2*0.02*M_obs*[1 0 0; 0 1 0; 0 0 1]*1;  % velocity
    Obs.NPO.vel_inject_time = 100;   % time in seconds to enable velocity injection - after Grip observer has converged.
    Obs.NPO.vel_inject = 1;    % use the position injection
    
    
    
    
    %Obs.RelVel.L_nu_r = 0.8*diag([2 1 0.001])*0.00002*0.01*M_obs; %diag( [1 1 0.00001])
    %Obs.RelVel.L_bias = diag([1,3,6])*0.1*Obs.RelVel.L_nu_r;
    
    Obs.RelVel.L_nu_r = 3*0.2*diag([2 0.8 0.001])*0.00002*0.01*M_obs; %diag( [1 1 0.00001])
    Obs.RelVel.L_bias = diag([1,2,6])*0.05*Obs.RelVel.L_nu_r;
    
    Obs.RelVel.L_Eta = 2*0.5*[Obs.NPO.omg_c 0 0;
        0 Obs.NPO.omg_c 0;
        0 0 Obs.NPO.omg_c];
    
    L_bias_eps = 0.09*[1;1;.03];
    L_biasGain_min = Obs.RelVel.L_bias;
    L_biasGain_max = 20*Obs.RelVel.L_bias;
    
elseif Sim.vessno == 2
    
   % Wave data. Peak frequencies, damping ratio and filter cut-off frequency.  Assuming JONSWAP spectrum.
    
    
    M_obs = inv(Obs.NPO.M_inv); 
    M_obs = diag(diag(M_obs));
    
    Obs.NPO.lmb   = 0.1;
    Obs.NPO.omg_0 = 4.66;
    Obs.NPO.damp = 1;
    Obs.NPO.k     = 1.22;
    NPO.mult_low_waves = 3.5; %multiplication of the K2-gain (position) when there are long waves
    %set to 1 if the normal wave tuning is wanted
    NPO.low_waves_threshold = 0.4; %if w_p is above this, then the K2 gain depending on w_c is increased 
    % \by the factor NPO.mult_low_waves
    NPO.no_waves_threshold = 0.3; %if w_p is above this, then the K2 gain depending on w_c is used
    % and if below, then a constant K2 is used
    K2_no_waves = 1*eye(3);
    Obs.NPO.omg_c = Obs.NPO.k * Obs.NPO.omg_0;  % cut-off frequency
    Obs.NPO.K3 = [10 0 0; 0 16 0; 0 0 3]*1;
    Obs.NPO.K4 = [Obs.NPO.K3(1,1)*5 0 0; 0 Obs.NPO.K3(2,2)*6 0; 0 0 Obs.NPO.K3(3,3)*7];
    
    
    
    % parameters that are not tuned for CS3 (just copied such that simulink
    % will run (parameters relevant for supply.m):
    
    % NPO with low frequency velocity estimate from Grip
    Obs.NPO.K41 = 0.01*M_obs*[1 0 0; 0 1 0; 0 0 2]*1;   % 0.02   % velocity
    Obs.NPO.K31 = 0.01*[Obs.NPO.K41(1,1) 0 0; 0 Obs.NPO.K41(2,2) 0; 0 0 Obs.NPO.K41(3,3)]; % 0.2   bias
    
    Obs.NPO.K22= Obs.NPO.omg_c*diag([1 1 1]);  % position
    Obs.NPO.K32 = 1*Obs.NPO.K31*diag([1 1 1]);  % bias
    Obs.NPO.K42 = 5*Obs.NPO.K41*diag([1 1 1]);  % velocity
    Obs.NPO.vel_inject_time = 50;   % time in seconds to enable velocity injection - after Grip observer has converged.
    Obs.NPO.vel_inject = 1;    % use the position injection
    
    
    
    
    %Obs.RelVel.L_nu_r = 0.8*diag([2 1 0.001])*0.00002*0.01*M_obs; %diag( [1 1 0.00001])
    %Obs.RelVel.L_bias = diag([1,3,6])*0.1*Obs.RelVel.L_nu_r;
    
    Obs.RelVel.L_nu_r = 3*0.2*diag([2 0.8 0.001])*0.00002*0.01*M_obs; %diag( [1 1 0.00001])
    Obs.RelVel.L_bias = diag([1,2,6])*0.05*Obs.RelVel.L_nu_r;
    
    Obs.RelVel.L_Eta = 2*0.5*[Obs.NPO.omg_c 0 0;
        0 Obs.NPO.omg_c 0;
        0 0 Obs.NPO.omg_c];
    
    L_bias_eps = 0.09*[1;1;.03];
    L_biasGain_min = Obs.RelVel.L_bias;
    L_biasGain_max = 20*Obs.RelVel.L_bias;
    
    
    Obs.Hybridgains.Buffersize = 100;
    Obs.Hybridgains.Bufferovl = 50;
end





%
% %******************************
% % Design parameters
%
% Obs.NPO.Aw = [0 0 0 1 0 0;
%              0 0 0 0 1 0;
%              0 0 0 0 0 1;
% -Obs.NPO.omg_0*Obs.NPO.omg_0   0   0 -2*Obs.NPO.lmb*Obs.NPO.omg_0    0  0;
% 0 -Obs.NPO.omg_0*Obs.NPO.omg_0  0  0 -2*Obs.NPO.lmb*Obs.NPO.omg_0      0;
%  0    0 -Obs.NPO.omg_0*Obs.NPO.omg_0  0  0 -2*Obs.NPO.lmb*Obs.NPO.omg_0];
%
%
% %******************************
% % Observer gain matrices - Fossen 2011 Handbook
%
% Obs.NPO.K1 = [-2*(Obs.NPO.damp-Obs.NPO.lmb)*(Obs.NPO.omg_c/Obs.NPO.omg_0) 0 0;
%                          0 -2*(Obs.NPO.damp-Obs.NPO.lmb)*(Obs.NPO.omg_c/Obs.NPO.omg_0) 0;
%                          0 0 -2*(Obs.NPO.damp-Obs.NPO.lmb)*(Obs.NPO.omg_c/Obs.NPO.omg_0);
%                       2*Obs.NPO.omg_0*(Obs.NPO.damp-Obs.NPO.lmb) 0 0;
%                       0 2*Obs.NPO.omg_0*(Obs.NPO.damp-Obs.NPO.lmb) 0;
%                       0 0 2*Obs.NPO.omg_0*(Obs.NPO.damp-Obs.NPO.lmb)];
% Obs.NPO.K2 = [Obs.NPO.omg_c 0 0;
%                          0 Obs.NPO.omg_c 0;
%                          0 0 Obs.NPO.omg_c];


%% Kinematic observer settings
% Note! Tune so that the attitude converges a lot faster than the linear
% dynamics, as the attitude is used there.


if Sim.vessno == 1
    
    % in attitude observer phi, theta, psi
    Obs.INS.Ki = [0.1 0 0; 0 0.1 0; 0 0 0.1];
    Obs.INS.Kp = [1 0 0; 0 1 0; 0 0 1];
    %Obs.INS.Ki = .001*[0.1 0 0; 0 0.1 0; 0 0 .1];
    %Obs.INS.Kp = 1*[1 0 0; 0 1 0; 0 0 1];
    
    % Gains for VVR and linear position and velocity observer from Bryne 2015
    
    % Vertical reference (VR) feedback gains only on heave
    Obs.INS.Kpipi = 0.4655;
    Obs.INS.Kppi = 0.1083;
    Obs.INS.Kvpi = 0.148;
    Obs.INS.Kbpi = 0.001;  % Kxipi in the paper Bryne 2015
    
    % in linear position and velocity observer
    % surge, sway and heave from the VR
    Obs.INS.K1 = 1*[0.419 0 0; 0 0.419 0; 0 0 Obs.INS.Kppi]; % FB gain on dot hat p (Kpp)
    Obs.INS.K2 = 1*[0.0878 0 0; 0 0.0878 0; 0 0 Obs.INS.Kvpi]; % FB gain on dot hat v (Kvp)
    Obs.INS.K3 = 1*[0.0091 0 0; 0 0.0091 0; 0 0 Obs.INS.Kbpi]; % FB gain on   (Kxip)
    
elseif Sim.vessno == 2
    % in attitude observer phi, theta, psi
    Obs.INS.Ki = 1*[0.1 0 0; 0 0.1 0; 0 0 0.1];
    Obs.INS.Kp = 2*[1 0 0; 0 1 0; 0 0 1];
    %Obs.INS.Ki = .001*[0.1 0 0; 0 0.1 0; 0 0 .1];
    %Obs.INS.Kp = 1*[1 0 0; 0 1 0; 0 0 1];
    
    % Gains for VVR and linear position and velocity observer from Bryne 2015
    
    % Vertical reference (VR) feedback gains only on heave
    Obs.INS.Kpipi = 0.4655;
    Obs.INS.Kppi = 0.1083;
    Obs.INS.Kvpi = 0.148;
    Obs.INS.Kbpi = 0.0001;  % Kxipi in the paper Bryne 2015
    
    % in linear position and velocity observer
    % surge, sway and heave from the VR
    Obs.INS.K1 = 1*[0.419 0 0; 0 0.419 0; 0 0 Obs.INS.Kppi]; % FB gain on dot hat p (Kpp)
    Obs.INS.K2 = 1*[0.0878 0 0; 0 0.0878 0; 0 0 Obs.INS.Kvpi]; % FB gain on dot hat v (Kvp)
    Obs.INS.K3 = 1*[0.0091 0 0; 0 0.0091 0; 0 0 Obs.INS.Kbpi]; % FB gain on   (Kxip)
    
    
end
%% Grip 2013 observer with VVr from Bryne et.al. 2015
% gains listed here are taken from Grip et.al 2013 (those in parenthesis from Bryne et. al. 2015)

%if Sim.vessno == 1
    
    % attitude observer
    Obs.Grip.k1 = 5*0.55*diag([1 1 1]);   %1*0.55;%0.55;%1;  % (0.55) influences yaw 5*
    %Obs.Grip.k2 = 0.55*diag([1 150 1]);   %1*0.55;%1.5; %(0.55) influences
    %roll and pitch 5*
    Obs.Grip.k2 = 5*0.55*diag([1 1 1]);   %1*0.55;%1.5; %(0.55) influences roll and pitch 5*
    %Obs.Grip.ki = 0.008*[1.2 0 0;0 1.1 0; 0 0 1.2];%0.008;  %(0.01)
    Obs.Grip.ki = 0.01*[1 0 0;0 1 0; 0 0 1];%0.008;  %(0.01)
    % M_hatb > Mb and ||hat b|| < Mb
    Obs.Grip.Mb = 5; % 1;  % Should be set to a value far above the presumed gyro bias value
    Obs.Grip.M_hatb = 10;  % 2;  % what should this value be? Mhatb > Mb
    
    
    % linear observer
    
    Obs.Grip.HG = 1; %2;  % (1)   % high-gain
    
    % gains in surge, sway
    %Obs.Grip.Kpp = 30*.419*[1 0; 0 1];   % 2 x 2 0.6 (0.419)
    %Obs.Grip.Kvp = 200*.0878*[1 0; 0 1.0];   % 2 x 2 0.11 (0.0878)
    %Obs.Grip.Kxip = 100*0.0091*[1 0; 0 1];   % 2 x 2 0.006 (0.0091)
    Obs.Grip.Kpp = 2*3*.419*[1 0; 0 1];   % 2 x 2 0.6 (0.419) 2*  3*
    Obs.Grip.Kvp = 2*3*.0878*[1 0; 0 1.0];   % 2 x 2 0.11 (0.0878) 2*  3*
    Obs.Grip.Kxip = 2*3*0.0091*[1 0; 0 1];   % 2 x 2 0.006 (0.0091) 2*  3*
    
    
    % Vertical reference (VR) feedback gains only on heave from Bryne et. al. 2015
    %Obs.Grip.Kpipi = 10*0.4655; % gain on the virtual heave reference
    %Obs.Grip.Kppi =10*0.1083; % gain in the position diff eq  
    %Obs.Grip.Kvpi = 10*0.0148; % Gain in the velocity diff eq.  
    %Obs.Grip.Kxipi = 10*0.0015;  %Gain in the bias diff eq
    Obs.Grip.Kpipi = 1*0.4655; % gain on the virtual heave reference
    Obs.Grip.Kppi =1*0.1083; % gain in the position diff eq  
    Obs.Grip.Kvpi = 1*0.0148; % Gain in the velocity diff eq.  
    Obs.Grip.Kxipi = 1*0.0015;  %Gain in the bias diff eq
 
    Obs.Grip.Init = 0*pi*[0 0 0];   %[0.75 0.5 -0.5];  %[0.25 0.2 1];  %  %[1 0.5 -1];  % Note: quaternion mapping non-unique for theta = pi
    
    
 %% Hybrid Grip 2013
 
 
Obs.HybridGrip.delta = 1;  %0.8*10^(-8); %0.004; does not make a difference no bias %0.01;good with bias  %0.0007;  % jump threshold
Obs.HybridGrip.DwellTime = 0.01;  % time between checking the objective function J(\hatq)
q = euler2q(Obs.Grip.Init(1),Obs.Grip.Init(2),Obs.Grip.Init(3));
Obs.HybridGrip.Init = q/(q'*q);

% Hybrid Grip take 2... adding noise to angles in jump map
% Gaussian noise generator - Euler angles - converted to q in Simulink
Obs.HybridGrip.mean = [0 0 0 0]';   % mean about the zero quaternion
Obs.HybridGrip.eps = 4;  %1.5;  %0.5; % 1;        % boundary around 180 deg
var = (0.1*Obs.HybridGrip.eps)^2;                 % want the noise large enough to push the estimates away from 180deg
Obs.HybridGrip.variance = [0.1*var,0.1*var,0.1*var,0.1*var]'; 
Obs.HybridGrip.seed = 42;

 
    
%% Grip 2015 - estimating the rotation matrix in stead of the attitude
% atitude observer
    Obs.Grip2015.Sigma = 1;    %  scalar > 0 Grip: 1
    Obs.Grip2015.Kp = 1*[1 0 0; 0 1 0; 0 0 1];       % tuning matrix Kp = Kp^\top > 0 Grip: eye(3)
    Obs.Grip2015.Rinit = eye(3);  % initial rotation matrix estimate
    Obs.Grip2015.ki = 0.01*[1 0 0;0 1 0; 0 0 1];%0.008;  %(0.01)
    Obs.Grip2015.Ma = 15;      % saturation on acceleration in the calculation of J hat
    Obs.Grip2015.mu = 0.05;  % small number to not divide by zero when converting Rhat to SO3 (not used usually)
    % M_hatb > Mb and ||hat b|| < Mb
    Obs.Grip2015.Mb = 0.5*pi/180; % 1;  % Should be set to a value far above the presumed gyro bias value
    Obs.Grip2015.M_hatb = 0.8*pi/180;  % 2;  % what should this value be? Mhatb > Mb
    
    
% linear observer
    Obs.Grip2015.HG = 1; %2;  % (1)   % high-gain  Note - high gain not used in Grip 2015
    
    
    Obs.Grip2015.Kpp = 3.3*eye(2);  %5*.419*[1 0; 0 1];   % 2 x 2 0.6 (0.419)
    Obs.Grip2015.Kvp = 1*2.74*eye(2);  %5*.0878*[1 0; 0 1.0];   % 2 x 2 0.11 (0.0878)
    Obs.Grip2015.Kxip = 0.0091*eye(2);  %5*0.0091*[1 0; 0 1];   % 2 x 2 0.006 (0.0091)

% VVR from Bryne 2015
    Obs.Grip2015.Kpipi = 1*0.4655; % gain on the virtual heave reference
    Obs.Grip2015.Kppi =10*0.1083; % gain in the position diff eq  
    Obs.Grip2015.Kvpi = 10*0.0148; % Gain in the velocity diff eq.  
    Obs.Grip2015.Kxipi = 0.0010;  %Gain in the bias diff eq
    
    
    
    
    
  
    
    %{
% attitude observer
Obs.Grip.k1 = 13*0.55*diag([1 1 1]);   %1*0.55;%0.55;%1;  % /0.55) influences yaw
Obs.Grip.k2 = 0.55*diag([0.9*1.3 1.5*150 1*1.8]);   %1*0.55;%1.5; %(0.55) influences roll and pitch
Obs.Grip.ki = 0.01*[1.2 0 0;0 1.1 0; 0 0 1.2];%0.008;  %(0.01)

% M_hatb > Mb and ||hat b|| < Mb
Obs.Grip.Mb = 10;  % Should be set to a value far above the presumed gyro bias value
Obs.Grip.M_hatb = 20;  % what should this value be?




% linear observer

Obs.Grip.HG = 1; %2;  % (1)   % high-gain

% gains in surge, sway
Obs.Grip.Kpp = 40*0.419*[1 0; 0 1.1];   % 2 x 2 0.6 (0.419)
Obs.Grip.Kvp = 100*.0878*[1 0; 0 1.0];   % 2 x 2 0.11 (0.0878)
Obs.Grip.Kxip = 100*0.0091*[1 0; 0 1];   % 2 x 2 0.006 (0.0091)



% Vertical reference (VR) feedback gains only on heave from Bryne et. al. 2015
Obs.Grip.Kpipi = 10*130*0.4655; % gain on the virtual heave reference
Obs.Grip.Kppi =10*130*0.1083; % gain in the position diff eq  1.5*
Obs.Grip.Kvpi = 10*130*0.0148; % Gain in the velocity diff eq.  2.8*
Obs.Grip.Kxipi = 10*130*0.0015;  %Gain in the bias diff eq

    %}
    
    
    % wave filter
    
    Obs.GripWF.Bn = [1 0]';          % B in notch filter part
    Obs.GripWF.Dn = 1;               % D in notch filter part
    Obs.GripWF.lamb_n = 1;           % damping in notch
    Obs.GripWF.zeta =0.05;           % notch damping coeff.  0.05 used by Bryne 2015, submitted, 0.01 is better for velocity -> but this is on perfectly known wp
    Obs.GripWF.Bw = [0 0 1]';        % B in wave-filter part
    Obs.GripWF.Cw = [0 1 1];         % C in wave-filter part
    Obs.GripWF.lamb = 0.1;           % damping in wave-filter part
    
    a = [  0    1.0000         0
        -Par.Wave.omega_peak(1)^2  -2*Par.Wave.omega_peak(1)*Obs.GripWF.lamb  0
        0         0         0];
    
    qx = 0.1*diag([0.01 6 0.1]);    % surge
    qy = 0.01*diag([0.1 6 0.1]);      % Sway
    qpsi = 0.01*diag([0.01 4 0.1]);       % yaw
    qu = 0.01*diag([0.01 4 0.5]);       % SURGE VEL
    qv = 0.01*diag([0.01 4 0.5]);       % sway vel
    
    % Prvd:
    % q = diag([0.01 5 0.01]);  ustabil, for hyt
    % q = diag([0.01 3 0.1]);   ikke s bra performance
    
    % innovation gains:
    [Xx,L,Obs.GripWF.Gx] = care(a',Obs.GripWF.Cw',qx');  % gain in surge
    [Xy,L,Obs.GripWF.Gy] = care(a',Obs.GripWF.Cw',qy');  % gain in sway
    [Xpsi,L,Obs.GripWF.Gpsi] = care(a',Obs.GripWF.Cw',qpsi');  % gain in yaw
    [Xu,L,Obs.GripWF.Gu] = care(a',Obs.GripWF.Cw',qu');  % gain in surge vel
    [Xv,L,Obs.GripWF.Gv] = care(a',Obs.GripWF.Cw',qv');  % gain in sway vel
    
    
    
    q = 0.01*diag([0.01 5 0.5]);
    % Prvd:
    % q = diag([0.01 5 0.01]);  ustabil, for hyt
    % q = diag([0.01 3 0.1]);   ikke s bra performance
    [X,L,G] = care(a',Obs.GripWF.Cw',q');
    
    Obs.GripWF.Kw = X*Obs.GripWF.Cw';
    
    Obs.GripWF.Gx = Obs.GripWF.Kw'*6;
    Obs.GripWF.Gy = Obs.GripWF.Kw'*6;
    Obs.GripWF.Gpsi = Obs.GripWF.Kw'*6;
    Obs.GripWF.Gu = Obs.GripWF.Kw'*4;
    Obs.GripWF.Gv = Obs.GripWF.Kw'*4;
    
    
    % q = 0.01*diag([0.01 5 0.5]);
    % Obs.GripWF.Kw = Obs.GripWF.Kw*6;
    
    
    % G = eye(3);
    % R = 0.1;
    %
    % [L1,P1,E1] = lqe(a,G,Obs.GripWF.Cw,Q,R)
    
    
    %
    
    %
    % X =
    %
    %     0.5342    0.0889   -0.0000
    %     0.0889    0.9259   -0.0000
    %    -0.0000   -0.0000    0.3162
    %
    %
    % L =
    %
    %   -0.0750 + 0.7462i
    %   -0.0750 - 0.7462i
    %   -0.3162 + 0.0000i
    %
    %
    % G =
    %
    %    -0.0000   -0.0000    0.3162
    %
    %
    % q = diag([0.01 4 0.5])
    %
    % X =
    %
    %     7.5347    0.0089    0.0000
    %     0.0089   13.3926   -0.0000
    %     0.0000   -0.0000    0.7071
    
%end
