%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
%
% Plots.m
%
% Project:	"Marine Cybernetics Simulator: MCSim Hybrid Sensor and Model Based"
%
% Abstract:	Plot parameters
% 
% Inputs:	-
%
% Outputs:	-
%	
%
% Author:       Svenn Are Vrn and Astrid Brodtkorb, Autumn 2015
%               Modifiend by Birte Steinsvik, Spring 2019
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
close all

%% INITIALIZE PLOT SIZES AND TIME
t = time_out; % Simulation time

% colors
mblue = [0.9290, 0.6940, 0.1250];
morange = [0.8500, 0.3250, 0.0980];
myellow = [0.9290, 0.6940, 0.1250];
mpurple = [0.4940, 0.1840, 0.5560];

 return
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Position meas + desired + supervisor      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1)
f = plot(t,sup,'k--',t,eta_des(:,1),t,eta_3DOF_meas(:,1),'LineWidth',1.5);
grid on
hold on
plot(t,eta_des(:,2),t,eta_3DOF_meas(:,2),'LineWidth',1.5)
plot(t,eta_des(:,3),t,eta_3DOF_meas(:,3),'LineWidth',1.5)
legend('Supervisor','\eta_{X_{des}}', '\eta_{X}', '\eta_{Y_{des}}', '\eta_{Y}','\eta_{\psi_{des}}', '\eta_{\psi}', 'location', 'Northeast')
ylabel('Position [m]')
xlabel('Time [s]')
hold off

return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\j';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig')


return
% robust test
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\robust_test\unb2_sw_tau1';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig')


return
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    REFERENCE MODELS: orig + unb + both       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(2)
plot(t,eta_des_orig(:,1),'color',myellow,'LineWidth',1.5)
ylim([0 3.5]);
grid on
hold on
plot(t, eta_des_unb(:,1),'color',morange,'LineWidth',1.5)
plot(t, eta_des(:,1),'k--','LineWidth',1.5)
legend('Original model','Unberthing model', 'Enabled model')
ylabel('Position [m]')
xlabel('Time [s]')
hold off

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tau_c, F_wind  , eta_3   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(33)
f = plot(t,sup,'k--',t,tau_c(:,1),'LineWidth',1.5); %t,tau_c(:,2), t,F_wind_out(:,2),
grid on
hold on
f = plot(t,F_wind_out(:,1),t,eta_3DOF_meas(:,3),'LineWidth',1.5);
legend('Supervisor','\tau_{c_X}','F_{wind_X}','location', 'Northwest' ) %, '\tau_{b_X}','\tau{_c_Y}', '\tau_{pid_Y}', '\tau_{b_Y}' )
ylabel('Forces [N]')
xlabel('Time [s]')
hold off

%return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\unb2_sw2_180_forces_x2';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig') 


return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\robust_test\unb2_tau1_taux';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig')
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tau_c  and tau_pid       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(3)
f = plot(t,sup,'k--',t,tau_c(:,1),t,tau_pid(:,1),'--','LineWidth',1.5);
grid on
%hold on
%plot(t,tau_pid(:,2),t,tau_b(:,2),t,tau_c(:,2),'--','LineWidth',1.5)
legend('Supervisor','\tau_{c_X}','\tau_{pid_X}') %, 'location', 'Northwest' ) %, '\tau_{b_X}','\tau{_c_Y}', '\tau_{pid_Y}', '\tau_{b_Y}' )
ylabel('Commanded thrust, \tau [N]')
xlabel('Time [s]')
%hold off

return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\UNB2_uten\unb2_sc3_65_taux';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig') 

return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\robust_test\unb2_tau17_taux';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig')

%% tau Y
figure(451)
f = plot(t,sup,'k--',t,tau_c(:,2),t,tau_pid(:,2),'--','LineWidth',1.5);
grid on
legend('Supervisor','\tau_{c_Y}','\tau_{pid_Y}') %, 'location', 'Southwest' )
ylabel('Commanded thrust, \tau [N]')
xlabel('Time [s]')

%return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\UNB2_uten\unb2_sc3_065_tauy';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig') 

return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\robust_test\unb2_tau17_tauy';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig')

return

%% Tau x + y
figure(350)
f = plot(t,sup,'k--',t,tau_c(:,1),'LineWidth',1.5);
grid on
hold on
f = plot(t,tau_c(:,2),'LineWidth',1.5);
grid on
legend('Supervisor','\tau_{c_X}','\tau_{c_Y}') %, 'location', 'Southwest' )
ylabel('Commanded thrust, \tau [N]')
xlabel('Time [s]')
hold off

%return
figname = 'C:\Users\birte\OneDrive\Documents\5. klasse\Master Thesis\Report\Figures\TEST' %UNB2_uten\unb2_sc3_06_tauxy';
saveas(gcf,figname,'epsc')
saveas(gcf,figname,'fig') 



%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      F_wind  , eta_3   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


figure(5)
f = plot(t,F_wind_out(:,1),t,F_wind_out(:,2),t,eta_3DOF_meas(:,3),'LineWidth',1.5);
grid on

legend('F_{wind_X}','F_{wind_Y}','\eta_{psi}', 'location', 'Northwest' ) %, '\tau_{b_X}','\tau{_c_Y}', '\tau_{pid_Y}', '\tau_{b_Y}' )
ylabel('Forces [N] // Angle [rad]')
xlabel('Time [s]')



%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      POSITION PLOTS MEASURED + OBSERVED      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(12);
subplot(3,1,1)
figure(12)
plot(t,eta_3DOF_meas(:,1),'b',t,NPO_eta_hat(:,1),'r',t,eta_des(:,1),'k--','LineWidth',2); %<- Specify plot properites
legend('Eta meas North','Eta LF','Eta hat NPO','Eta desired',  'Location', 'NorthEast');
ylabel('Position [m]')
xlabel('Time [s]')
% xlim([1480 2000])
%ylim([-0.2 1.1])
title('Eta North');
subplot(3,1,2)
plot(t,eta_3DOF_meas(:,2),'b',t,NPO_eta_hat(:,2),'r',t,eta_des(:,2),'k--','LineWidth',2); %<- Specify plot properites
%legend('Eta meas yaw','Eta LF','Eta hat NPO East','Eta desired',  'Location', 'NorthEast');
%ylim([-0.12 0.46])
% xlim([1480 2000])
ylabel('m')
title('Eta East');
subplot(3,1,3)
plot(t,(180/pi)*eta_3DOF_meas(:,3),'b',t,(180/pi)*NPO_eta_hat(:,3),'r',t,(180/pi)*eta_des(:,3),'k--','LineWidth',2); %<- Specify plot properites
legend('Eta meas','Eta hat model-based','Eta desired',  'Location', 'NorthEast');
xlabel('Time [s]');
% xlim([1480 2000])
ylabel('deg')
title('Eta yaw'); 

figure(13)
plot(eta_3DOF_meas(:,2),eta_3DOF_meas(:,1),'b',NPO_eta_hat(:,2),NPO_eta_hat(:,1),'r',eta_des(:,2),eta_des(:,1),'k--','LineWidth',2);
xlabel('East [m]');
ylabel('North [m]');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   END OF POSITION PLOTS MEASURED + OBSERVED (ALL)     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

return

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          VELOCITY PLOTS LF + OBSERVED        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(22);
subplot(3,1,1)
plot(t,nuLF(:,1),'b',t,NPO_nu_hat(:,1),'r','LineWidth',2); %<- Specify plot properites
legend('\nu LF surge','\nu hat NPO surge',  'Location', 'NorthEast');
ylabel('m')
%xlim([0 60])
%ylim([-0.2 1.1])
title('Surge velocity');
subplot(3,1,2)
plot(t,nuLF(:,2),'b',t,NPO_nu_hat(:,2),'r','LineWidth',2); %<- Specify plot properites
legend('\nu LF sway','\nu hat NPO sway',  'Location', 'NorthEast');
%ylim([-0.12 0.46])
ylabel('m')
title('Sway velocity');
subplot(3,1,3)
plot(t,nuLF(:,6),'b',t,NPO_nu_hat(:,3),'r','LineWidth',2); %<- Specify plot properites
legend('\nu LF yaw','\nu hat NPO yaw',  'Location', 'NorthEast');
xlabel('Time [s]');
ylabel('rad')
title('Yaw velocity');


