%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
%
% ReferenceModel.m
%
% Project:	"Marine Cybernetics Simulator: MCSim Hybrid Sensor and Model Based"
%
% Abstract:	Here the relevant parameters are set for a position reference model as decribed in Chapter 10.2.1 of Fossen (2011). A third order linear
% reference model is implemented, and saturating elements for maximum
% velocity and acceleration is included.
% 
% Inputs:	-
%
% Outputs:	-
%	
%
% Author:       Svenn Are Vrn and Astrid Brodtkorb, Autumn 2015
%               Altered for autonomous docking by Birte L. Steinsvik, spring 2019 
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

% if supply.m:
if Sim.vessno == 1
    Control.Vessel.pos_init = [Sim.Eta_init(1),Sim.Eta_init(2),Sim.Eta_init(6)];                 % Initial  setpoint for vessel

    %Control.Vessel.step_time1 = 1500;                      % Sets step time for the change in desired position.
    %Control.Vessel.pos_d1 = [20;20;-90*(pi/180)];                    % Sets desired setpoint for vessel. Station-keeping when set to zero.
    Control.Vessel.pos_d1 = [(Quay.Pos(1));(Quay.Pos(2));Quay.Pos(3)];        % Sets desired setpoint for vessel. Station-keeping when set to zero.
    %Control.Vessel.pos_d1 = [(Quay.Pos(1)+0.1);(Quay.Pos(2)+0.1);Quay.Pos(3)];
    
    Control.Vessel.pos_unb = [Control.Vessel.pos_d1(1)-0.5,...
                       Control.Vessel.pos_d1(2)-0.5, Control.Vessel.pos_d1(3)];        %Desired setpoint 0.5 m from quay, used for reference signal during unberthing
%     
%   Control.Vessel.pos_unb = [Control.Vessel.pos_d1(1)-0.1,...
%                         Control.Vessel.pos_d1(2)-0.1, Control.Vessel.pos_d1(3)];        %Desired setpoint 0.5 m from quay, used for reference signal during unberthing
   
                    
    %Control.Vessel.step_time2 = 5000;                      % Sets step time for the change in desired position.
    %Control.Vessel.pos_d_additional = [20;20;0];                    % Sets additional (adds to the previous) desired setpoint for vessel. Station-keeping when set to zero.
    Control.Vessel.pos_d2 = [0; 0; 0*Quay.Pos(3)+(pi/2)];
    
      
         Ramp.StartTime = 60000;                           % Sets ram time for a change in desired speed (used for testing so far)
         Ramp.slope = [.3;.2;0];                           % desired speed of the vessel (N,E,yaw) 
    
    % Reference model settings, only for use with DP

    Control.Vessel.max_vel_d = [10 10 5*3.14/180];    % Sets desired maximum velocity for vessel in surge, sway and yaw
    Control.Vessel.max_acc_d = [10 10 10*pi/180];    % Sets desired maximum acceleration for vessel in surge, sway and yaw


    % Design parameters in reference model

    ReferenceMdl.RelDamping = [1.2 1.2 1.2];           % Relative damping in reference model (Stiffness = Omega.^2)
    ReferenceMdl.Omega = [0.15 0.15 0.15];       % Omega vector in reference model (Damping = 2 RelDamping * Omega)
    ReferenceMdl.TC   = [80 80 100];             % Time constants in surge, svai and yaw for the first order LP filter to get the filtered reference signal
    
    % if CS 3:

elseif Sim.vessno == 2
% Desired positions and desired heading, [x y psi] and step time
Control.Vessel.pos_init = [Sim.Eta_init(1),Sim.Eta_init(2),Sim.Eta_init(6)];                                      % Initial  setpoint for vessel

    Control.Vessel.pos_d1 = [(Quay.Pos(1));(Quay.Pos(2));Quay.Pos(3)];        % Sets desired setpoint for vessel. Station-keeping when set to zero.
    %Control.Vessel.pos_d1 = [(Quay.Pos(1)+0.1);(Quay.Pos(2)+0.1);Quay.Pos(3)];
    
    if Unb == 1
    Control.Vessel.pos_unb = [Control.Vessel.pos_d1(1)-0.1,...
                       Control.Vessel.pos_d1(2)-0.1, Control.Vessel.pos_d1(3)];        %Desired setpoint 0.1 m from quay, used for reference signal during unberthing UNB1
    end

    if Unb == 2
    Control.Vessel.pos_unb = [Control.Vessel.pos_d1(1)-0.5,...
                       Control.Vessel.pos_d1(2)-0.5, Control.Vessel.pos_d1(3)];        %Desired setpoint 0.5 m from quay, used for reference signal during unberthing UNB2
     end
   
                   
 Control.Vessel.pos_d2 = [0; 0; Quay.Pos(3)+(pi/2)];  % Final set-point of simulation
 
         Ramp.StartTime = 60000;                           % Sets ram time for a change in desired speed (used for testing so far)
         Ramp.slope = [.3;.2;0];                           % desired speed of the vessel (N,E,yaw) 

% Reference model settings, only for use with DP

Control.Vessel.max_vel_d = [.15 .1 3*3.14/180];    % Sets desired maximum velocity for vessel in surge, sway and yaw
Control.Vessel.max_acc_d = [0.2 0.2 10*pi/180];    % Sets desired maximum acceleration for vessel in surge, sway and yaw


% Design parameters in reference model

ReferenceMdl.RelDamping = [1.2 1.2 1.2];           % Relative damping in reference model (Stiffness = Omega.^2)
ReferenceMdl.Omega = [0.15 0.15 0.15];       % Omega vector in reference model (Damping = 2 RelDamping * Omega)
ReferenceMdl.TC   = [40 40 30];             % Time constants in surge, svai and yaw for the first order LP filter to get the filtered reference signal

end
% Design matrixes in reference model with respect to the paramterers set above

% Design matrix Af (with respect to time constants).

ReferenceMdl.Af =      [1/ReferenceMdl.TC(1) 0 0;         
                                    0 1/ReferenceMdl.TC(2) 0;          
                                    0 0 1/ReferenceMdl.TC(3)];



% Design spring force matrix in reference model (with respect to omega vector).                               
ReferenceMdl.Stiffness =  [ReferenceMdl.Omega(1)^2 0 0;                                   
                                    0 ReferenceMdl.Omega(2)^2 0;                                       
                                    0 0 ReferenceMdl.Omega(3)^2];
                                
                                
                                % Design Damp matrix in reference model (with respect to omega vector and relative damping).                                
ReferenceMdl.Damping =    [2*ReferenceMdl.RelDamping(1)*ReferenceMdl.Omega(1) 0 0;     
                                    0 2*ReferenceMdl.RelDamping(2)*ReferenceMdl.Omega(2) 0;    
                                    0 0 2*ReferenceMdl.RelDamping(3)*ReferenceMdl.Omega(3)];