%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
% 
% SensorModule.m
%
% Project:	"Marine Cybernetics Simulator: MCSim Hybrid Sensor and Model Based"
%
% Abstract:	Run this script to set the sensor parameters
% 
% Inputs:	-
%
% Outputs:	-
%	
%
% Author:       Svenn Are Vrn and Astrid Brodtkorb, Autumn 2015
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 




Sim.Sensors				= 0;			% 1 for including sensor models, 0 for not.


% Values based on Kongsberg Maritime MRU 5+ and DPS 200 (www.km.kongsberg.com)
% measurement noise (Markov process)

if Sim.vessno == 1
% GPS: position measurement [surge sway] -> DOUBLE CHECK VALUES!
Sensor.GPS.Bias_atm = [0 0]';   % bias from the atmosphere
Sensor.GPS.Var_multipath = 1*[0.1 0.1]'.^2;  % variance
Sensor.GPS.Pow_thermal =  [0 0]';     %meters white noise power
Sensor.GPS.Sample_time = 0.01; % are they used anywherE?

% sample times used in the Main module, with all the observer and
% controller (not in the SensorModule block. This should be cleaned up):
Sensor.GPS.SampleTime = 1;
Sensor.Compass.SampleTime = 0.1;


% Compass: [yaw] -> DOUBLE CHECK VALUES
Sensor.Compass.Bias = 0;
Sensor.Compass.Var = 1*((pi/180)*0.01)^2;               % radians
Sensor.Compass.Sample_time = 0.01;


% Gyro: angular rates [roll pitch yaw]
Sensor.Gyro.Bias = 5*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro.Var = 1*((pi/180)*[0.025 0.025 0.025]').^2;   % rates in rad/s   Values based on Kongsberg Maritime MRU 5 (5+ has in roll, pitch: 0.001 0.001)
Sensor.Gyro.Sample_time = 0.01;

% If needed, the MRU can give roll and pitch angles as well as rates

% Linear velocity noise is set to zero, but if it is to be used somewhere,
% we need to add (large) noise values here...


% IMU: linear acceleration  [surge sway heave]
Sensor.IMU.Bias = 0*[0.1 0.2 0.3]';   % DO NOT USE!
Sensor.IMU.Var = 1*([0.002 0.002 0.002]').^2;  % m^2/s    Values based on Kongsberg Maritime MRU 5
Sensor.IMU.Sample_time = 0.01;



% Anemometer: wind [magnitude angle] -> DOUBLE CHECK VALUES!
Sensor.Anemometer.Bias = [0 0]'; 
Sensor.Anemometer.Var = ([0.1 0.1]').^2;   % magnitude variance, angle variance
Sensor.Anemometer.Sample_time = 0.01; 


elseif Sim.vessno == 2
    
    
    % GPS: position measurement [surge sway] -> DOUBLE CHECK VALUES!
Sensor.GPS.Bias_atm = [0 0]';   % bias from the atmosphere
Sensor.GPS.Var_multipath = 1*[0.1/30 0.1/30]'.^2;  % variance
Sensor.GPS.Pow_thermal =  [0 0]';     %meters white noise power
Sensor.GPS.Sample_time = 0.01; % are they used anywherE? THis is probably not used

% sample times used in the Main module, with all the observer and
% controller (not in the SensorModule block. This should be cleaned up):
Sensor.GPS.SampleTime = 1;  %1/sqrt(30);
Sensor.Compass.SampleTime = 0.1;  %0.1/sqrt(30);


% Compass: [yaw] -> DOUBLE CHECK VALUES
Sensor.Compass.Bias = 0;
Sensor.Compass.Var = 1*((pi/(180*30))*0.01)^2;               % radians
Sensor.Compass.Sample_time = 0.01;


% Gyro: angular rates [roll pitch yaw]
Sensor.Gyro.Bias = 0*5*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro.Var = 1*((pi/180)*[0.025 0.025 0.025]').^2;   % rates in rad/s   Values based on Kongsberg Maritime MRU 5 (5+ has in roll, pitch: 0.001 0.001)
Sensor.Gyro.Sample_time = 0.01;

% If needed, the MRU can give roll and pitch angles as well as rates

% Linear velocity noise is set to zero, but if it is to be used somewhere,
% we need to add (large) noise values here...


% IMU: linear acceleration  [surge sway heave]
Sensor.IMU.Bias = 0*[0.1 0.2 0.3]';   % DO NOT USE!
Sensor.IMU.Var = 1*([0.002 0.002 0.002]').^2;  % m^2/s    Values based on Kongsberg Maritime MRU 5
Sensor.IMU.Sample_time = 0.01;



% Anemometer: wind [magnitude angle] -> DOUBLE CHECK VALUES!
Sensor.Anemometer.Bias = [0 0]'; 
Sensor.Anemometer.Var = 0*([0.1 0.1]').^2;   % magnitude variance, angle variance
Sensor.Anemometer.Sample_time = 0.01; 

end
