%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
% Parameters.m
%
% Project:	"Marine Cybernetics Simulator: MCSim"
%
% Abstract:	Simulation parameters, to be edited by user
%
% Inputs:	-
%
% Outputs:	-
%
% Calls:	-
%
% Author:	yvind Smogeli, Autumn 2002
%
% Changes:	Continuously by OS
%
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


% Global constants
global Glob
Glob.g 					= 9.81;			% Constant of gravity
Glob.rhow				= 1025;			% Density of seawater
Glob.p_0				= 101325;		% Atmospheric pressure
Glob.p_w				= 1230;			% Evaporation pressure of water
Glob.epsilon			= 1E-3;			% Small number

% Model reduction matrix
Glob.H36 = [1 0 0 0 0 0; 0 1 0 0 0 0 ; 0 0 0 0 0 1];

% Initial values for LF position, velocity and acceleration.
Sim.Eta_init			= [0 0 0 0 0 0*(pi/180)];
Sim.Nu_init				= [0 0 0 0 0 0];
Sim.Nu_d_init			= [0 0 0 0 0 0];

%----------------------------------------------------------------------------------------------------------------------------------
%% Simulation settings.

% You can choose to run both LF and WF or either one alone. You may
% also choose which excitation forces to use.
Sim.WF_model 			= 1;			% 1 for including WF model, 0 for not
Sim.LF_model 			= 1;			% 1 for including LF model, 0 for not
Sim.Prescribed_motion 	= 0;			% 1 for prescribing the vessel motion in Simulink (edit the file). ONLY FOR DEBUGGING!

% Load modules
Sim.Current_load		= 0;			% 1 for including current load model (= nonlinear damping), 0 for not
Sim.Wind_load			= 1;			% 1 for including wind load model, 0 for not
Sim.Wavedrift_load		= 0;			% 1 for including wavedrift load model, 0 for not.
Sim.Ice_load            = 0;            % 1 for including ice load model


% DEBUGGING OPTIONS, all set to 1 for normal simulations
Sim.C_rb				= 1;			% 1 for including C_rb (rigid-body coriolis-centripetal forces), 0 for not
Sim.C_a					= 1;			% 1 for including C_a (added mass coriolis-centripetal forces), 0 for not
Sim.D_l					= 1;			% 1 for including D_l (linear damping forces), 0 for not
Sim.G					= 1;			% 1 for including G (restoring forces), 0 for not
Sim.Nu_r_C_a			= 1;			% 1 for using Nu_r in C_a, 0 for using Nu
Sim.Nu_r_D_l			= 1;			% 1 for using Nu_r in D_l, 0 for using Nu

% Reduced sampling rates for time-consuming routines, 1 second = 1 Hz should be adequate for most, but less for the thruster module
Sim.WF_sample_time		= 0.01;			% Sample time for the WF model in seconds
Sim.WD_sample_time		= 0.01;			% Sample time for the Wavedrift load model in seconds
Sim.currload_sample_time= 0.01;			% Sample time for the current load model in seconds
Sim.windload_sample_time= 0.01;			% Sample time for the wind load model in seconds
Sim.iceload_sample_time = 0.01;            % sample time for the ice load in seconds
Sim.gust_sample_time	= 0.01;			% Wind gust model sample time in seconds
Sim.t_wave_sample_time	= 0.01;			% Thruster wave calculation sample time


% External force, for testing response, for debugging
Sim.Ext_force 				= [0 0 0 0 0 0];	% Force and moment to be applied in vessel coordinate system
Sim.Ext_force_coord			= [0 0 0];			% Coordinates in vessel coordinate system at which force is applied
Sim.Ext_force_T				= 20;				% Time constant for ramping up external force


% Enabling/disabling control system
Sim.Observer	= 1;            % 1 for using pbserver estimates in closed loop, 0 for using ideal LF motion in closed loop
Sim.Control		= 1;			% 1 for enabling control system, 0 for disabling


% Observer settings
Sim.Vessel.Observer.wave_filter   	= 1;       % 1 for enabling wave filtering in observer, 0 for disabling
Sim.Vessel.Observer.bias_time     	= 0;       % 1 for including bias time constant in observer, 0 for T = 0

%----------------------------------------------------------------------------------------------------------------------------------
% Settings for simplified thruster model, only for Sim.Thrusters.enable == 1
Sim.Thrusters.enable = 1;                     % 1 for enabling simplified thuster module, 0 for no thruster dynamics
Sim.Thrusters.thrust_sat	= 1;			% 1 for enabling saturation of thrust vector in simple thruster dynamics, 0 for not
Sim.Thrusters.thrust_time	= 1;			% 1 for enabling 1st order thruster dynamics, 0 for not

% Max thrust vector. Only used when Sim.Thrusters.thrust_sat == 1
%Sim.Thrusters.Tau_max		= [10 10 12];

% Time constants for 1st order thruster system, when Sim.Thrusters.thrust_time == 1

if Sim.vessno == 1
    Sim.Thrusters.T_surge		= 5;		% s
    Sim.Thrusters.T_sway		= 5;		% s
    Sim.Thrusters.T_yaw			= 5;		% s
elseif Sim.vessno == 2  % CS3
    Sim.Thrusters.T_surge		= .5;		% s
    Sim.Thrusters.T_sway		= .5;		% s
    Sim.Thrusters.T_yaw			= .5;		% s
end


%----------------------------------------------------------------------------------------------------------------------------------
% Set all control options and parameters in Controls.m
%----------------------------------------------------------------------------------------------------------------------------------

%----------------------------------------------------------------------------------------------------------------------------------
%% Environment settings

% Waves
% Number of wave components = nfreq*ndir, but using the rand_dir and energylim options
% will reduce the number significantly. A large number of wave components slows the
% simulation down significantly, but for a realistic sea state you should use at least 100.
Par.Wave.spectrum		= 2;			% Spectrum type: 1 = ITTC, 2 = JONSWAP, 3 = Doubly Peaked

Par.Wave.time_change(1) = 5000; %200;          % Time to begin the change the wave
Par.Wave.time_change(2) = 5000; %200.53;          % Time to end the change the wave
for i=3:2:48
    Par.Wave.time_change(i)   = Par.Wave.time_change(i-2)+439;  %91.30;          % Time to begin the change the wave
    Par.Wave.time_change(i+1) = Par.Wave.time_change(i-1)+439;   %91.30;          % Time to end the change the wave
end

% % Modelscale
Par.Wave.hs(1)			= 0.04;			% Significant wave heigth in sea state
Par.Wave.omega_peak(1)	= 4.33;        	% Peak frequency component in sea state, 0 for expectation value based on Hs
for i=2:25
    Par.Wave.hs(i)		    = Par.Wave.hs(i-1)+0.0120833;			% Significant wave heigth in sea state
    Par.Wave.omega_peak(i)  = Par.Wave.omega_peak(i-1)-0.09125;         	% Peak frequency component in sea state, 0 for expectation value based on Hs
end

% % Fullscale
% Par.Wave.hs(1)			= 6; % 8;%6;%6;  %7.0;	%2;		% Significant wave heigth in sea state
% Par.Wave.omega_peak(1)	= 0.75;  %0.53;%.43;  %0.75;  %0.8;  % 0.5;  %.75;   % saet peak frequency very low when no waves     	% Peak frequency component in sea state, 0 for expectation value based on Hs
% for i=2:25
%     Par.Wave.hs(i)		    = Par.Wave.hs(i-1)+0.41668;%0.01708333333333;			% Significant wave heigth in sea state
%     Par.Wave.omega_peak(i)  = Par.Wave.omega_peak(i-1)-0.010;         	% Peak frequency component in sea state, 0 for expectation value based on Hs
% end

Par.Wave.psi_mean		= 90*pi/180;     % Mean direction of sea state  For 0 vessel heading 180 is head incident.
Par.Wave.spread			= 2;			% Spreading factor for direction spectrum. 1 recommended by ITTC, 2 by ISSC. High value centers wave energy
Par.Wave.gamma			= 3.3;			% Gamma value for JONSWAP spectrum, if using this. Default is 3.3

Par.Wave.nwave  	    = 10;	%100		% Number of waves <=nfreq*ndir = max 1200
Par.Wave.nfreq  	    = 5;	%50		% Number of frequency components in simulation
Par.Wave.ndir			= 5;			% Number of wave directions in simulation
Par.Wave.rand_freq		= 1;			% 1 for random frequencies
Par.Wave.rand_dir		= 1;			% 1 for random directions
Par.Wave.cutoff 		= 2.5;			% Cutoff frequency for spectrum = cutoff*omega_peak
Par.Wave.cutoff_dir		= 0;			% Cutoff direction for spectrum = cutoff_dir+-omega_peak


% Wind
Par.Wind.steptime       = 180;          % Steptime [s] for change in wind velocity and direction
Par.Wind.u10			= 0; %-1;  %0*5;% 1 hour mean wind velocity at 10 meters. -1 for using the expectation value based on Hs and surface drag coefficient (also changes the direction)
Par.Wind.u10_2          = 2.7;            % change wind velocity after steptime
Par.Wind.dir			= 180*pi/180;     % Mean wind direction (rad). Set equal to mean wave direction if vel = -1  for 0 vessel heading 180 is head incident.
Par.Wind.dir_2          = (180)*pi/180;     % change wind direction aftre steptime
Par.Wind.gust			= 0;			% 1 for including wind gust, 0 for not
Par.Wind.spectrum		= 2;			% 1 for Harris wind spectrum, 2 for NORSOK wind spectrum
Par.Wind.kappa			= 0.0026;		% Sea surface drag coefficient for calculation of mean wind speed. Harris wind spectrum: kappa = 0.0026.
Par.Wind.kappa_H		= 0.0026;		% Sea surface drag coefficient Harris wind spectrum.
Par.Wind.L				= 1800;			% Scaling length for Harris wind spectrum
Par.Wind.n 				= 0.468;		% NORSOK spectrum parameter
Par.Wind.nfreq			= 20;			% Number of frequency components in gust realization
Par.Wind.minfreq		= 1E-4;			% Minimum frequency in gust realization [Hz]
Par.Wind.maxfreq		= 1E-1;			% Maximum frequency in gust realization [Hz]
Par.Wind.var_dir		= 0;			% 1 for variation of direction, 0 for constant
Par.Wind.mju_dir		= 0;			% Gauss Markov process constant for varying direction. 0 for random walk.
Par.Wind.sat_dir		= 5*pi/180;		% Max variation in wind direction: direction = [dir - sat_dir, dir + sat_dir]
Par.Wind.wn_t_dir 		= 0.5;			% Direction white noise sample time
Par.Wind.wn_p_dir 		= 0.000005;		% Direction white noise power
Par.Wind.wn_s_dir 		= 6375;			% Direction white noise seed

% Wind velocity - expectation value based on Hs
if Par.Wind.u10 == -1
	Par.Wind.u10 = 1.764 + 3.426*Par.Wave.hs(1)^0.78; % Mean wind speed at 10 meters
	Par.Wind.dir = Par.Wave.psi_mean;
	disp(' ')
	disp(sprintf('Calculated mean wind velocity at 10 meters (u10):  %0.2f m/s',Par.Wind.u10))
	fprintf('Calculated mean wind direction: %0.0f degrees\n',Par.Wind.dir*180/pi)
end


% Current
% This is just for surface velocity. Define the current profile including depth below as Par.Current.Profile
% NB! DEFINE THE MEAN CURRENT DIRECTION AS FUNCTION OF TIME TO FACILITATE
% FAST CHANGES
Par.Current.vel 		= 0*0.01;			% Current velocity (m/s)
Par.Current.dir			= 180*pi/180;	% Current direction (rad)   for 0 vessel heading 180 is head incident.
Par.Current.var_vel		= 1;			% 1 for variation of velocity, 0 for constant
Par.Current.var_dir		= 1;			% 1 for variation of direction, 0 for constant
Par.Current.mju_vel		= 0;			% Gauss Markov process constant for varying velocity. 0 for random walk.
Par.Current.sat_vel		= 0.2;			% Max variation in current velocity: velocity = [vel - sat_vel, vel + sat_vel]
Par.Current.wn_t_vel 	= 0.5;			% Velocity white noise sample time
Par.Current.wn_p_vel 	= 0.000001;		% Velocity white noise power
Par.Current.wn_s_vel 	= 1234;			% Velocity white noise seed
Par.Current.mju_dir		= 0;			% Gauss Markov process constant for varying direction. 0 for random walk.
Par.Current.sat_dir		= 5*pi/180;		% Max variation in current direction: direction = [dir - sat_dir, dir + sat_dir]
Par.Current.wn_t_dir 	= 0.5;			% Direction white noise sample time
Par.Current.wn_p_dir 	= 0.000001;		% Direction white noise power
Par.Current.wn_s_dir 	= 4321;			% Direction white noise seed


Par.Current.vel2		= 0; % 0.05*0.5;			% Current velocity (m/s)
Par.Current.dir2		= 90*pi/180;	% Current direction (rad)   for 0 vessel heading 180 is head incident.

Current.Change.Time =5000; % time for a changing current (ad hoc solution for simulating a
% wave train ect.
TC_currentChange = 30; % time constant for the current change
% Define current profile [xvel, yvel, depth]. If only using surface component, it is fully defined by the surface velocity and direction
Par.Current.Profile = ...
    [Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 0;
    Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 5];

Par.Current.Profile2 = ...
    [Par.Current.vel2*[cos(Par.Current.dir2), sin(Par.Current.dir2)], 0;
    Par.Current.vel2*[cos(Par.Current.dir2), sin(Par.Current.dir2)], 5];
% Level ice
% Model the ice as  equivalent current velocities, and use the current coefficients to approximate
% ice loads  on the vessel.
% AHBrodtkorb, SAVrn 2015
% NB! DEFINE THE MEAN CURRENT DIRECTION AS FUNCTION OF TIME TO FACILITATE
% FAST CHANGES

Sim.Ice_load_tstart = 10000;  % time to go into ice
Sim.Ice_load_tstop = 15000;  % time to go out of ice


Par.Ice.vel 		= 0;			% Mean ice flow velocity (m/s)
Par.Ice.dir			= 135*pi/180;	% Mean ice flow direction (rad)  for 0 vessel heading 180 is head incident.
Par.Ice.NoisePower = Par.Ice.vel*10^9;          % Noise on the calculated ice force - to emulate very rapid changes

Par.Ice.vel_pulseamp = 4;               % Ice pulse veloctiy amplitude
Par.Ice.vel_pulseperiod = 80;             % width of pulse in seconds
Par.Ice.vel_pulsewidth = 44;           % pulsewidth as percent of period
Par.Ice.vel_pulsephase = 0.01;     % phase in radians

Par.Ice.dir_pulseamp =  50*(pi/180);  % Ice pulse direction amplitude [rad]
Par.Ice.dir_pulseperiod  = 50;      % width of pulse in seconds
Par.Ice.dir_pulsewidth = 47;    % pulsewidth as percent of period
Par.Ice.dir_pulsephase  = 0.67;   % phase in radians


Par.Ice.var_vel		= 1;			% 1 for variation of velocity, 0 for constant
Par.Ice.var_dir		= 1;			% 1 for variation of direction, 0 for constant
Par.Ice.mju_vel		= 500;			% Gauss Markov process constant for varying velocity. 0 for random walk.
Par.Ice.sat_vel		= 1;			% Velocity: velocity = [vel - sat_vel, vel + sat_vel]
Par.Ice.wn_t_vel 	= 0.5;			% Velocity white noise sample time
Par.Ice.wn_p_vel 	= 10;           % Velocity white noise power
Par.Ice.wn_s_vel 	= 1234;			% Velocity white noise seed
Par.Ice.mju_dir		= 0;			% Gauss Markov process constant for varying direction. 0 for random walk.
Par.Ice.sat_dir		= 5*pi/180;    % Max variation in direction: direction = [dir - sat_dir, dir + sat_dir]  [rad]
Par.Ice.wn_t_dir 	= 0.5;			% Direction white noise sample time
Par.Ice.wn_p_dir 	= 0.1;          % Direction white noise power
Par.Ice.wn_s_dir 	= 1234;			% Direction white noise seed


%----------------------------------------------------------------------------------------------------------------------------------
% Definitions
Par.Wave.spectypes{1} 	= 'ITTC';
Par.Wave.spectypes{2} 	= 'JONSWAP';
Par.Wave.spectypes{3} 	= 'Doubly Peaked';
Par.Wind.spectypes{1}	= 'Harrison';
Par.Wind.spectypes{2}	= 'NORSOK';

Sim.Sensor.Types{1}		= 'Off';
Sim.Sensor.Types{2}		= 'On';

Sim.Sensor.Feedback{1}	= 'WF + LF';
Sim.Sensor.Feedback{2}	= 'LF only';

Sim.VessObs.Types{1}	= 'Disabled, using measurement in feedback';
Sim.VessObs.Types{2}	= 'Enabled, but using measurement in feedback';
Sim.VessObs.Types{3}	= 'Enabled, using output in feedback';
