% Scale down to model-scale - CS3


% Contents of vessel and vesselABC structs
%    vessel.headings:                     headings
%    vessel.velocities:                   velocities
%    vessel.freqs:                        frequencies (A and B matrices)
%
%    vessel.A(6,6,freqno,velno):          added mass matrix
%    vessel.B(6,6,freqno,velno):          damping matrix
%    vessel.C(6,6,freqno,velno):          spring stiffness matrix
%    vessel.MRB(6,6):                     rigid-body mass matrix


% Scale of model
lambda =  1/30; 
% Scaling matrix for scaling mass and damping matrices
Lambda = [lambda^3*ones(3), lambda^4*ones(3);
          lambda^4*ones(3), lambda^5*ones(3)];

      
      
% Main particulars
Modelvessel.main.Lpp = lambda*vessel.main.Lpp;
Modelvessel.main.Lwl = lambda*vessel.main.Lwl;
Modelvessel.main.B = lambda*vessel.main.B;
Modelvessel.main.T = lambda*vessel.main.T;


M_RB_m = [lambda^3,lambda^3,lambda^3,lambda^5,lambda^5,lambda^5 ].*vessel.MRB;
Modelvessel.MRB = M_RB_m;

% Hydrodynamic coefficients


% Geometric scaling Lpp_modelscale = lambda*L_pp_fullscale
% Froude scaling Fn_fullscale = Fn_modelscale
% Sverre Steen, Experimental methods Lecture notes 2014, page 11

% Structural mass [kg] scales with lambda^3
% Force [N = kg*m/s^s] scales with lambda^3
% Length [m] scales with lambda
% Moment [Nm] scales with lambda^4
% Time [s] scales with sqrt(lambda)
% Acceleration [m/s^2] is not scaled

% Moment of inertia [kg*m^2] scales with lambda^5
% Structural damping coefficient (D*nu) [N = kg/s * m/s] scales with lambda^3
% Structural damping coefficient (D*nu) [Nm = kgm/s * m/s] scales with lambda^5

M_A0_m = Lambda.*vesselABC.MA_0;
Sym_A0 = 0.5*(M_A0_m + M_A0_m')

Modelvessel.MA_0 = Sym_A0;

M_Ainf_m = Lambda.*vesselABC.MA_inf;
Sym_Ainf = 0.5*(M_Ainf_m + M_Ainf_m')
Modelvessel.MA_inf = Sym_Ainf;


D0_m = Lambda.*vesselABC.B_0;  % vessel.B(:,:,35,5);
Sym_D0 = 0.5*(D0_m + D0_m')
Modelvessel.B_0 = Sym_D0;

Dinf_m = Lambda.*vesselABC.B_inf;
Sym_Dinf = 0.5*(Dinf_m + Dinf_m')
Modelvessel.B_inf = Sym_Dinf;

Bv_m = Lambda.*vessel.Bv
Modelvessel.Bv = Bv_m;



%% Local variable names for inertia and damping

% Rigid body inertia and added mass
Mrb = [Modelvessel.MRB(1,1), 0 , 0 ;
    0, Modelvessel.MRB(2,2), Modelvessel.MRB(2,6);
    0, Modelvessel.MRB(6,2) ,Modelvessel.MRB(6,6)];

if bool_mode == 1
    Ma = [ Modelvessel.MA_0(1,1), 0 , 0 ;
        0,  Modelvessel.MA_0(2,2),  Modelvessel.MA_0(2,6);
        0,  Modelvessel.MA_0(6,2), Modelvessel.MA_0(6,6)];  % zero frequency for DP
else
    Ma = [ Modelvessel.MA_inf(1,1), 0 , 0 ;
        0,  Modelvessel.MA_inf(2,2),  Modelvessel.MA_inf(2,6);
        0,  Modelvessel.MA_inf(6,2), Modelvessel.MA_inf(6,6)]; % infinite frequency for maneuvering
    
end

M = Mrb + Ma;

Minv = inv(M);

% Linear potential damping and viscous damping - Fossen p. 115
if bool_mode == 1
    D = [Modelvessel.B_0(1,1) + Modelvessel.Bv(1,1), Modelvessel.B_0(1,2),Modelvessel.B_0(1,6);
        Modelvessel.B_0(2,1), Modelvessel.B_0(2,2)+ Modelvessel.Bv(2,2),Modelvessel.B_0(2,6);
        Modelvessel.B_0(6,1), Modelvessel.B_0(6,2),Modelvessel.B_0(6,6)+ Modelvessel.Bv(6,6)];% zero frequency for DP
else
    D = [Modelvessel.B_inf(1,1) + Modelvessel.Bv(1,1), Modelvessel.B_inf(1,2),Modelvessel.B_inf(1,6);
        Modelvessel.B_inf(2,1), Modelvessel.B_inf(2,2)+ Modelvessel.Bv(2,2),Modelvessel.B_inf(2,6);
        Modelvessel.B_inf(6,1), Modelvessel.B_inf(6,2),Modelvessel.B_inf(6,6)+ Modelvessel.Bv(6,6)];% infinite frequency for maneuvering
end

